\name{bestMclust}
\alias{bestMclust}
\alias{projMclust}
\alias{envMclust}
\title{Choose 'best' clusters}
\description{
  
  Chooses the 'best' regression cluster(s), if the number of true
  clusters is known.

}

\usage{

  bestMclust(clust, nc=1, crit="value")
  projMclust(clust, x, y)
  envMclust(clust, x, y, dist=0)

}

\arguments{

  \item{clust}{Cluster object returned by \code{oregMclust} or \code{circMclust}.}
  \item{nc}{Number of 'best' clusters.}
  \item{crit}{Name of the column to determine the best clusters.}
  \item{x,y}{Original observations.}
  \item{dist}{Maximal distance of observation from cluster center.}

}
\details{
  \code{oregMclust} and \code{circMclust} return a matrix containing not
  only the parameters of the found clusters but the value of the heights
  of the corresponding local maxima as well as how often each cluster is
  found. Both are reasonable criteria for choosing 'best' clusters, which
  can be done by \code{bestMclust}.
  Additional criteria could be the number of observations projected to
  each cluster or the number of observations lying in a certain
  neighbourhood of the cluster center point.

  \code{projMclust} adds a column \code{proj} to \code{clust} which
  contains the number of points belonging to each cluster in the sense
  that each observation belongs to the cluster with shortest orthogonal
  distance. If \code{clust} is comming from \code{circMclust} a second
  column \code{projrel} is added which contains this number relativ to
  the radius of the particular circle.

  \code{envMclust} adds a column \code{env} to \code{clust} which
  contains the number of observations lying in a
  \code{dist}-neighbourhood of each cluster center. If \code{clust} is
  comming from \code{circMclust} a second column \code{envrel} is added
  which contains this number relativ to the radius of the particular
  circle.
}
\value{
  Both functions return a matrix of clusters.
}
\references{
  Mller, C.H., Garlipp, T. (2003) Simple consistent cluster methods
  based on redescending M-estimators with an application to edge
  identification in images, to appear in \emph{JMVA}.
}
\author{ Tim Garlipp, garlipp@mathematik.uni-oldenburg.de }
\keyword{ cluster }
\keyword{ regression }
\keyword{ robust }

\eof
\name{circMclust}
\alias{circMclust}
\alias{plot.circMclust}
\alias{print.circMclust}
\title{Circular Clustering}
\description{
  Computation of cluster center points for circular regression data. A
  cluster method based on redescending M-estimators is used.
}

\usage{
  circMclust(datax, datay, bw,
             method="const", prec=4,
             minsx=min(datax),            maxsx=max(datax), nx=10,
             minsy=min(datay),            maxsy=max(datay), ny=10,
             minsr=0.01*max(datax,datay),
             maxsr=(max(datax,datay)-min(datax,datay)), nr=10,
             nsc=5,         nc=NULL, 
             minsd=NULL,    maxsd=NULL,
             brminx=minsx,  brmaxx=maxsx,
             brminy=minsy,  brmaxy=maxsy,
             brminr=minsr,  brmaxr=maxsr,
             brmaxit=1000)

  plot.circMclust(x, datax, datay,
                  ccol="black", clty=1, clwd=3, ...)

  print.circMclust(x, ...)
}

\arguments{
  \item{datax, datay}{numerical vectors of coordinates of the observations.}
  \item{bw}{positive number. Bandwidth for the cluster method.}
  \item{method}{optional string. Method of choosing starting values for
    maximization. Possible values are:
    \itemize{
      \item "const": a constant number of circles is used. By default,
      \code{nx}*\code{ny} equidistant midpoints within the range of the
      observations with \code{nr} different radiuses are uses as
      starting circles. The domain of the midpoints and radiuses can
      optionally be given by [\code{minsx},\code{maxsx}],
      [\code{minsy},\code{maxsy}], and [\code{minsr},\code{maxsr}].
      \item "all": every circle through any three observations is used.
      \item "prob": Clusters are searched iteratively with randomly
      chosen starting circles until either no new clusters are found
      (default), or until \code{nc} clusters are found. The precision of
      distinguishing the clusters can be tuned with the parameter
      \code{prec}. In each iteration \code{nc} times a circle through
      three randomly chosen observations is used as starting
      value. With the parameters \code{minsd} and \code{maxsd} the
      minimal and maximal distance of these observations could be
      limited.
    }
  }
  \item{nx, ny}{optional positive integer. Number of starting midpoints,
    if \code{method="const"} is chosen.}
  \item{nr}{optional positive integer. Number of starting radiuses, if
    \code{method="const"} is chosen.}
  \item{prec}{optional positive integer. Tuning parameter for
    distinguishing different clusters, which is passed to
    \code{\link{deldupMclust}}.}
  \item{minsx, maxsx, minsy, maxsy, minsr}{optional numbers
    determining the domain of starting midpoints and the range of
    radiuses, if \code{method="const"} is chosen.}
  \item{maxsr}{optional number determining the maximum radius used as
    starting value. Note, that this is valid for all \code{method}s
    while \code{minsx}, \code{maxsx}, \code{minsy}, \code{maxsy}, and
    \code{minsr} are only used by \code{method="const"}.}
  \item{nsc}{optional positive integer. Number of starting circles in each
    iteration, if \code{method="prob"} is chosen.}
  \item{nc}{optional positive integer. Number of clusters to search, if
    \code{method="prob"} is chosen. Note that, if \code{nc} is to
    large, e.g. \code{nc} clusters cannot be found, the function does not
    terminate. Attention! Using Windows, the routine cannot even be
    breaked manually in this case!}
  \item{minsd, maxsd}{optional positive numbers. Minimal and maximal
    distance of starting points which are used, if \code{method="prob"}
    is chosen.}
  \item{brminx, brmaxx, brminy, brmaxy, brminr, brmaxr}{optional
    numbers. The maximization is stopped, if the midpoint leaves the
    domain [\code{brminx},\code{brmaxx}]x[\code{brminy},\code{brmaxy}] or
    if the radius leaves [\code{brminr},\code{brmaxr}].}
  \item{brmaxit}{optional positive integer. Since the maximization could
    be very slow in some cases depending on the starting value, the
    maximization is stopped after \code{brmaxit} iterations.}
  
  \item{x}{object returned by \code{circMclust}}
  
  \item{ccol,clty,clwd}{optional graphic parameters used for plotting
    the circles.}
  \item{...}{Additional parameters passed to \code{plot}.}
}

\details{
  \code{circMclust} implements a cluster method using local
  maxima  of redescending M-estimators for the case of circular
  regression. This method is based on a method introduced by Mller and
  Garlipp in 2003 (see references).
  
  See also \code{\link{bestMclust}}, \code{\link{projMclust}}, and
  \code{\link{envMclust}} for choosing the 'real' clusters out of the
  found.
}

\value{
  Return value is a numerical matrix containing one row for every found
  cluster circle. The columns "cx" and "cy" are their midpoints and "r"
  are the radiuses.
  
  The columns "value" and "count" give the value of the objective
  function and the number, how often they are found.
}

\references{
  Mller, C.H., Garlipp, T. (2003) Simple consistent cluster methods
  based on redescending M-estimators with an application to edge
  identification in images, to appear in \emph{JMVA}.
}

\author{ Tim Garlipp, garlipp@mathematik.uni-oldenburg.de }

\seealso{\code{\link{bestMclust}}, \code{\link{projMclust}},
  \code{\link{envMclust}}, \code{\link{deldupMclust}}}

\examples{
  z <- (1:100 * pi)/50
  x <- c(sin(z) * 10 + 20,sin(z) * 30 + 80)+rnorm(200,0,2)
  y <- c(cos(z) * 10 + 20,cos(z) * 30 + 80)+rnorm(200,0,2)

  circ <- circMclust(x,y,5,method="prob",prec=1,nsc=20,minsd=10,maxsd=40)
  bestMclust(circ,2)
  plot(bestMclust(circ,2),x,y)
}

\keyword{ cluster }
\keyword{ regression }
\keyword{ robust }

\eof
\name{deldupMclust}
\alias{deldupMclust}
\title{Delete duplicate found clusters}
\description{
  Delete clusters differing only by rounding errors or having
  maximization value zero.
}

\usage{
  deldupMclust(clust, prec=NULL, ncol=NULL, dz=TRUE)
}

\arguments{
  \item{clust}{numerical matrix, which columns contain the parameters of
    the clusters.}
  \item{prec}{optional positive integer. Number of decimal places for
    rounding.}
  \item{ncol}{Number of columns describing the clusters.}
  \item{dz}{optional boolean. With \code{dz=TRUE} clusters are deleted,
    for which the objective function has value 0.}
}

\details{
  Since clusters found by \code{oregMclust} or \code{circMclust} often
  differ only by rounding errors, the function
  \code{deldupMclust} can be used for rounding and deleting
  duplicates. If \code{clust} has a column named \code{count} their
  values are summed appropriate. Otherwise such a column is added.
  
  For parameter \code{clust} the object returned from \code{oregMclust}
  or \code{circMclust} can be used. Alternatively, an arbitrary matrix
  can be given, of which the first \code{ncol} columns describe the
  clusters.
  The parameter \code{prec} is the number of decimal places for
  rounding; the default is no rounding.
  With \code{ncol} the number of columns, which describe the clusters,
  can be given. This is not needed, if \code{clust} is an object
  returned from \code{oregMclust} or \code{circMclust}.
}

\value{
  Return value is an object of the same type as \code{clust}.
}

\references{
  Mller, C.H., Garlipp, T. (2003) Simple consistent cluster methods
  based on redescending M-estimators with an application to edge
  identification in images, to appear in \emph{JMVA}.
}
\author{ Tim Garlipp, garlipp@mathematik.uni-oldenburg.de }

\seealso{\code{\link{oregMclust}}, \code{\link{circMclust}}}

\keyword{ cluster }
\keyword{ regression }
\keyword{ robust }

\eof
\name{edgecluster}
\alias{edgecluster}
\title{Edge detection in noisy images}
\description{
  \code{edgecluster} is a simple combination of \code{\link{edgepoints}}
  and \code{\link{oregMclust}}. It just passes the results of
  \code{\link{edgepoints}} to \code{\link{oregMclust}}.
}

\usage{
  edgecluster(data, h1n, h2n,
              maxval,
              bw         = max(h1n,h2n)/qnorm(0.975),
              asteps     = 4,
              estimator  = "M_median",
              kernel     = "gauss",
              score      = "gauss",
              sigma      = 1,
              kernelfunc = NULL)
}

\arguments{
  \item{data}{See description of \code{\link{edgepoints}}.}
  \item{h1n, h2n}{See description of \code{\link{edgepoints}}.}
  \item{asteps}{See description of \code{\link{edgepoints}}.}
  \item{estimator}{See description of \code{\link{edgepoints}}.}
  \item{kernel}{See description of \code{\link{edgepoints}}.}
  \item{score}{See description of \code{\link{edgepoints}}.}
  \item{sigma}{See description of \code{\link{edgepoints}}.}
  \item{kernelfunc}{See description of \code{\link{edgepoints}}.}
  
  \item{maxval}{See description of \code{\link{eplist}}.}

  \item{bw}{See description of \code{\link{oregMclust}}. See default
    gives the observations within the '\code{edgecluster}-window' 95\% of
    the weight.}
}

\value{
  Return value is a list of two numerical matrices. The first matrix
  contains the results of \code{\link{oregMclust}}, which are the
  'edgeclusters'. The second matrix contains the result of
  \code{\link{edgepoints}}.
}

\author{ Tim Garlipp, garlipp@mathematik.uni-oldenburg.de }

\seealso{\code{\link{edgepoints}}, \code{\link{oregMclust}},
  \code{\link{eplist}}}

\examples{
  y <- matrix(rep(0,60*60),nrow=60)
  y[21:40,21:40]<-1
  y <- y + matrix(rnorm(60*60,0,0.2),nrow=60)
  image(y,col=gray(seq(0,1,1/255)))

  ec <- edgecluster(y,0.05,0.05,0.7,
                    estimator="M_median",kernel="gauss")
  plot(bestMclust(ec[[1]],4),ec[[2]],xlim=c(0,1),ylim=c(0,1))
}

\keyword{ robust }
\keyword{ multivariate }
\keyword{ nonparametric }

\eof
\name{edgepoints}
\alias{edgepoints}
\title{Edge detection in noisy images}
\description{
  Detection of edgepoints by the difference of two rotated and
  asymmetric Kernel- or M-Kernel-Estimators.
}

\usage{
  edgepoints(data, h1n, h2n,
             asteps     = 4,
             estimator  = "kernel",
             kernel     = "mean",
             score      = "gauss",
             sigma      = 1,
             kernelfunc = NULL,
             margin     = FALSE)
}

\arguments{
  \item{data}{numerical matrix describing the noisy image.}
  \item{h1n, h2n}{positive numbers. Bandwidth for the kernels.}
  \item{asteps}{optional positive integer. Number of different angles used.}
  \item{estimator}{optional string. Estimator used within the
    windows. Possible values are:
    \itemize{
      \item "kernel": Kernel-Estimators. The used kernel
      function can be choosen by \code{kernel}.
      \item "M\_mean": M-Kernel-Estimators with mean as starting value.
      The used kernel function can be choosen by \code{kernel}, the
      score function can be choosen by \code{score}.
      \item "M\_median": M-Kernel-Estimators with median as starting value.
      The used kernel function can be choosen by \code{kernel}, the
      score function can be choosen by \code{score}.
      \item "median": Median, what is a special M-Kernel-Estimator.
      \item "test\_mean": Multiple Test for equal means in both windows
      for every angle.
      \item "test\_median":  Multiple Test for equal means in both windows
      for every angle.
    }
  }
  \item{kernel}{optional string. Kernel function if
    \code{estimator="kernel"}, \code{estimator="M_mean"}, or
    \code{estimator="M_median"}. Possible values are:
    \itemize{
      \item "mean": Rectangular kernel. With \code{estimator="kernel"}
      this gives an ordinary mean estimator. With
      \code{estimator="M_mean"} or \code{estimator="M_median"} this
      gives an M-Estimator.
      \item "linear": Linear kernel function. The distance of the
      observations to the common midpoint of both windows is linearly
      measured.
      \item "linear2": Linear kernel function. The distance of the
      observations to the midpoint of the window they belong to is
      linearly measured.
      \item "gauss": Density of the normal distribution with
      sd=0.5 and zero outside [-1,1]x[-1,1].
      \item "func": Arbitrary kernel function given by \code{kernelfunc}.
    }
  }
  \item{score}{optional string. Score function for M-Kernel-Estimators
    if \code{estimator="M_mean"} or
    \code{estimator="M_median"}. Possible values are:
    \itemize{
      \item "gauss": negativ density of the normal distribution. The
      deviation can be given by parameter \code{sigma}.
      \item "huber": The Huber score function is the absolute value
      (median) within an interval [-c,c] and the square function (mean)
      outside this interval. The value of c can be given by the
      parameter \code{sigma}.
    }
  }
  \item{sigma}{optional positiv number. Parameter for the score function
    \code{"gauss"} or \code{"huber"}.}
  \item{kernelfunc}{optional function taking two numbers as arguments
    and returning a positive number. Used as kernelfunction if
    \code{kernel="func"} is given. Note that the function should be zero
    outside [-1,1]x[-1,1] and that only one function is given for both
    windows. The 'lower' part of the domain, e.g. [-1,1]x[-1,0], is used
    within one window while the 'upper' part is used within the other.}
  \item{margin}{Optional value. Results near the margin are in
    general not very reasonable. Setting \code{margin=TRUE} they are
    calculated nevertheless. With \code{margin=FALSE} the returned
    matrices have the same dimension as \code{data} but the jump heights at
    the margin are set to zero. Setting \code{margin="cut"} the retuned
    matrices are cutted down by the margins. The default is
    \code{margin=FALSE}.}
}

\details{
  \code{edgepoints} implements several versions of the RDKE method,
  introduced by Qiu in 1997.
  The original method, which uses kernel estimates, is a generalized
  version, which uses M-Kernel-Estimators, and two test procedures. The
  test procedures are multiple tests for different angles for the
  hypothesis, that there are equal means (or medians) in both windows.
  All methods apply rotating and scaling in the correct order (see
  Garlipp, 2004).
}

\value{
  Return value is a list of two numerical matrices. The first matrix
  contains the maximal jump height for every pixel, if the choosen
  estimator is not a test procedure, and p-values otherwise. The second
  matrix contains the angle, which leads to the maximal jump heigth or
  minimal p-value.
}

\references{
  Garlipp, T. (2004), On Robust Jump Detection in Regression Surface
  with Applications to Image Analysis,
  \emph{Carl-von-Ossietzky-Universitt Oldenburg, Dissertation}
  
  Qiu, P. (1997), Nonparametric Estimation of Jump Surface, \emph{The
  Indian Journal of Statistics}, 59A, No.2, 268-294.
}

\author{ Tim Garlipp, garlipp@mathematik.uni-oldenburg.de }

\seealso{\code{\link{eplist}}}

\examples{
  y <- matrix(rep(0,60*60),nrow=60)
  y[21:40,21:40]<-1
  y <- y + matrix(rnorm(60*60,0,0.2),nrow=60)
  image(y,col=gray(seq(0,1,1/255)))

  ye <- edgepoints(y,0.05,0.05,estimator="M_median",kernel="gauss")
  image(ye[[1]]>0.7,col=gray(c(1,0)))
}

\keyword{ robust }
\keyword{ multivariate }
\keyword{ nonparametric }

\eof
\name{eplist}
\alias{eplist}
\title{Conversion of Matrices returned by edgepoints}
\description{
  The matrices returned by \code{\link{edgepoints}} are converted into a
  list of edgepoints and a list of corresponding angles.
  This is useful for processing the results of \code{\link{edgepoints}}
  by \code{\link{oregMclust}}.
}

\usage{
  eplist(data, maxval, test=FALSE, xc=NULL, yc=NULL)
}

\arguments{
  \item{data}{list object returned from \code{\link{edgepoints}}.}
  \item{maxval}{positive numbers. Critical value for deciding, if an
    pixel belongs to an edge or not.}
  \item{test}{optional boolean. Must be set to \code{TRUE} if
    \code{edgepoints} was used with \code{estimator="test_mean"} or
    \code{estimator="test_median"}. Then \code{maxval} is the level of
    the test.}
  \item{xc,yc}{optional numerical vectors defining the coordinates of
    the edgepoints. A pixel with jump height data[[1]][i,j] gets the
    coordinates (xc[i],yc[j]). As default, the coordinates are assumed as
    equidistant within [0,1], e.g. if the matrix of jump heights is a
    n-m-matrix, the pixel at position i,j gets the coordinates
    (i/n,j/m).}
}

\value{
  A numerical matrix is returned. The first two columns contain the
  coordinates of the pixels, for which the detected jump height is
  larger than \code{maxval} (or smaller then \code{maxval}, if
  \code{test=TRUE}). The third column contains the corresponding angles.
}

\author{ Tim Garlipp, garlipp@mathematik.uni-oldenburg.de }

\seealso{\code{\link{edgepoints}}}

\keyword{ robust }
\keyword{ multivariate }
\keyword{ nonparametric }

\eof
\name{oregMclust}
\alias{oregMclust}
\alias{regparm}
\alias{plot.oregMclust}
\alias{print.oregMclust}
\title{Orthogonal Regression Clustering}
\description{
  Computation of center points for regression data by orthogonal
  regression. A cluster method based on redescending M-estimators is
  used.
}

\usage{
  oregMclust(datax, datay, bw,
             method="const",
             xrange=range(datax), yrange=range(datay), prec=4,
             na=1, sa=NULL, nl=10, nc=NULL, brmaxit=1000)

  regparm(reg)

  plot.oregMclust(x, datax, datay, prec=3,
                  rcol="black", rlty=1, rlwd=3, ...)

  print.oregMclust(x, ...)
}

\arguments{
  \item{datax,datay}{numerical vectors of coordinates of the
    observations. Alternativly, a matrix with two or three columns can
    be given. The first two columns are interpreted as coordinates of
    the observations and, if available, the third is passed to parameter
    \code{sa}.}
  \item{bw}{positive number. Bandwidth for the cluster method.}
  \item{method}{optional string. Method of choosing starting values for
    maximization. Possible values are:
    \itemize{
      \item "const": a constant number of angles for every observation
      is used. By default, one horizontal line through any observation
      is used as starting value. If \code{na} is given, \code{na} lines
      through any observation are used. Alternatively, with the
      parameter \code{sa} a proper starting angle for every observation can
      be specified. In this case, \code{na} is ignored. The length of
      \code{sa} must be the number of observations.
      \item "all": every line through any two observations is used.
      \item "prob": Clusters are searched iteratively with randomly
      chosen starting values until either no new clusters are found
      (default), or until \code{nc} clusters are found. The precision of
      distinguishing the clusters can be tuned with the parameter
      \code{prec}. In each iteration \code{nl} times a line through two
      randomly chosen observations is used as starting value.
    }
  }
  \item{xrange, yrange}{optional numerical intervals describing the
    domains of the observations. This is only used for normalization of the
    data. Note, that both intervals should have approximately the same
    length or should be transformed otherwise. This is not done
    automatically since this transformation affects the choice of the
    bandwidth.}
  \item{prec}{optional positive integer. Tuning parameter for
    distinguishing different clusters, which is passed to
    \code{\link{deldupMclust}}.}
  \item{na}{optional positive integer. Number of angles per observation
    used as starting values if \code{method="const"} is chosen (default).} 
  \item{sa}{optional numerical vector. Angles (within [0,2pi)) used as
    starting values if \code{method="const"} is chosen (default).}
  \item{nl}{optional positive integer. Number of starting lines in each
    iteration, if \code{method="prob"} is chosen.}
  \item{nc}{optional positive integer. Number of clusters to search, if
    \code{method="prob"} is chosen. Note that, if \code{nc} is to
    large, e.g. \code{nc} clusters cannot be found, the function does not
    terminate. Attention! Using Windows, the routine cannot even be
    breaked manually in this case!}
  \item{brmaxit}{optional positive integer. Since the maximization could
    be very slow in some cases depending on the starting value, the
    maximization is stopped after \code{brmaxit} iterations.}

  \item{reg,x}{Object returned from \code{oregMclust}.}

  \item{rcol,rlty,rlwd}{optional graphic parameters used for plotting
    regression lines.}
  \item{...}{Additional parameters passed to \code{plot}.}
}
\details{
  \code{oregMclust} implements a cluster method based on redescending
  M-estimators for the case of orthogonal regression. This method is
  introduced by Mller and Garlipp in 2003 (see references).

  \code{regparm} transforms the columns "alpha" and "beta" to
  "intersept" and "slope".

  See also \code{\link{bestMclust}}, \code{\link{projMclust}}, and
  \code{\link{envMclust}} for choosing the 'real' clusters out of the
  found.
}

\value{
  Return value is a numerical matrix containing one row for every found
  regression center line. The Columns "alpha" and "beta" are their
  parameters in the representation (cos(alpha),sin(alpha))(x,y)' =
  beta, where alpha is within [0,2pi). For representation y=mx+b,
  the return value can be passed to \code{regparm}.
  
  The columns "value" and "count" give the value of the objective
  function and the number, how often they are found.
}
\references{
  Mller, C.H., Garlipp, T. (2003) Simple consistent cluster methods
  based on redescending M-estimators with an application to edge
  identification in images, to appear in \emph{JMVA}.
}

\author{ Tim Garlipp, garlipp@mathematik.uni-oldenburg.de }

\seealso{\code{\link{bestMclust}}, \code{\link{projMclust}},
  \code{\link{envMclust}}, \code{\link{deldupMclust}}}

\examples{
  x <- c(rnorm(100,0,3),rnorm(100,5,3))
  y <- c(-2*x[1:100]-5,0.5*x[101:200]+30)/2
  x <- x + rnorm(200,0,0.5)
  y <- y + rnorm(200,0,0.5)

  reg <- oregMclust(x,y,1,method="prob")
  reg <- projMclust(reg,x,y)
  reg
  plot(bestMclust(reg,2,crit="proj"),x,y)
}

\keyword{ cluster }
\keyword{ regression }
\keyword{ robust }

\eof
